"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
//@ts-ignore
var logger_1 = require("express-gateway/lib/logger");
var Keycloak = require("keycloak-connect");
var session = require("express-session");
var createMemoryStore = require("memorystore");
var memoryStore = createMemoryStore(session);
var DEFAULT_KEYCLOAK_PLUGIN_SETTINGS = {
    session: {
        secret: 'kc_secret',
        resave: false,
        saveUninitialized: true,
    },
    keycloakConfig: {},
    registerName: 'keycloak-protect',
    paths: ['/'],
};
exports.DefaultKeycloakPluginSettings = DEFAULT_KEYCLOAK_PLUGIN_SETTINGS;
var keycloakPlugin = {
    version: '1.2.0',
    init: function (ctx) {
        // this is slightly dodgy casting, as they don't expose settings on the public interface - but not sure how else you can access custom settings for a plugin
        var sessionStore = new memoryStore();
        var rawSettings = 
        //@ts-ignore
        ctx.settings;
        var sessionSettings = __assign(__assign(__assign({}, DEFAULT_KEYCLOAK_PLUGIN_SETTINGS.session), rawSettings.session), { store: sessionStore });
        var keycloakConfig = __assign(__assign({}, DEFAULT_KEYCLOAK_PLUGIN_SETTINGS.keycloakConfig), rawSettings.keycloakConfig);
        var pluginSettings = {
            //@ts-ignore
            session: sessionSettings,
            keycloakConfig: keycloakConfig,
            registerName: rawSettings.registerName || DEFAULT_KEYCLOAK_PLUGIN_SETTINGS.registerName,
            paths: rawSettings.paths || DEFAULT_KEYCLOAK_PLUGIN_SETTINGS.paths,
        };
        var keycloak = new Keycloak({ store: sessionStore }, pluginSettings.keycloakConfig);
        var logger = logger_1.createLoggerWithLabel('[EG:plugin:' + pluginSettings.registerName + ']');
        logger.debug('Init', pluginSettings.registerName);
        logger.info("Initialized Keycloak Plugin with settings: " + JSON.stringify(pluginSettings, null, '\t'));
        keycloak.authenticated = function (req) {
            var keyR = req;
            var grant = keyR.kauth.grant;
            logger.info('-- Keycloak Authenticated: ' + JSON.stringify(grant.access_token.content, null, '\t'));
        };
        keycloak.accessDenied = function (req, res) {
            logger.debug('Denied', res);
            logger.warn('-- Keycloak Access Denied.');
            res.status(403).end('Access Denied');
        };
        // setup our keycloak middleware
        ctx.registerGatewayRoute(function (app) {
            logger.debug('Register', ctx);
            logger.info('Registering Keycloak Middleware');
            app.use(pluginSettings.paths, session(pluginSettings.session));
            app.use(pluginSettings.paths, keycloak.middleware());
        });
        ctx.registerPolicy({
            name: pluginSettings.registerName,
            schema: {
                $id: 'http://express-gateway.io/schemas/policies/keycloak-protect.json',
                type: 'object',
                properties: {
                    role: {
                        description: 'the keycloak role to restrict access to',
                        type: 'string',
                    },
                    jsProtectTokenVar: {
                        description: 'the keycloak token variable name to reference the token in jsProtect',
                        type: 'string',
                    },
                    jsProtect: {
                        description: 'a js snippet to apply for whether a user has access.',
                        type: 'string',
                    },
                },
            },
            policy: function (actionParams) {
                logger.debug('policy', pluginSettings.registerName);
                logger.info("-- Keycloak Protect: " + JSON.stringify(actionParams, null, '\t'));
                //@ts-ignore
                if (pluginSettings.keycloakConfig['bearer-only']) {
                    return keycloak.protect();
                }
                if (actionParams.jsProtect) {
                    return keycloak.protect(function (token, req) {
                        //@ts-ignore
                        req.egContext[actionParams.jsProtectTokenVar || 'token'] = token;
                        //@ts-ignore
                        var runResult = req.egContext.run(actionParams.jsProtect);
                        logger.info('-- Keycloak Protect JS Result: ' + runResult);
                        return runResult;
                    });
                }
                return keycloak.protect(actionParams.role);
            },
        });
    },
    schema: {
        $id: 'http://express-gateway.io/schemas/plugin/keycloak.json',
        type: 'object',
        properties: {
            registerName: {
                title: 'Registring name',
                description: 'Multi keycloak feature',
                type: 'string',
            },
            paths: {
                title: 'Paths to apply protection',
                description: 'url paths to apply protection',
                type: 'array',
            },
            session: {
                title: 'Session Settings',
                description: 'Session Settings as outlined by express middleware',
                type: 'object',
            },
            keycloakConfig: {
                title: 'Keycloak Configuration',
                description: 'This can be used rather than requiring keycloak.json to be present',
                type: 'object',
            },
        },
    },
};
exports.KeycloakPlugin = keycloakPlugin;
exports.default = keycloakPlugin;
//# sourceMappingURL=KeycloakPlugin.js.map